import { isNil } from "lodash";
class RefPromiseHelper {
    constructor() {
        this.refMap = new Proxy({}, {
            set: (obj, prop, value) => {
                const valueProxy = new Proxy(value, {
                    set: (_obj, _prop, _value) => {
                        _obj[_prop] = _value;
                        if (!isNil(_value))
                            this.doResolveFunc(prop);
                        return true;
                    },
                });
                if (!isNil(value.current))
                    this.doResolveFunc(prop);
                obj[prop] = valueProxy;
                return true;
            },
        });
        this.resolveFuncMap = {};
        this.rejectTimerMap = {};
        this.doResolveFunc = (key) => {
            var _a, _b;
            (_a = this.resolveFuncMap[key]) === null || _a === void 0 ? void 0 : _a.forEach(item => item === null || item === void 0 ? void 0 : item());
            Reflect.deleteProperty(this.resolveFuncMap, key);
            (_b = this.rejectTimerMap[key]) === null || _b === void 0 ? void 0 : _b.forEach(item => item && clearTimeout(item));
            Reflect.deleteProperty(this.rejectTimerMap, key);
        };
        this.addListener = (key, timeout) => {
            return new Promise((resolve, reject) => {
                var _a;
                if (!isNil((_a = this.refMap[key]) === null || _a === void 0 ? void 0 : _a.current))
                    resolve(this.refMap[key].current);
                else {
                    if (isNil(this.refMap[key]))
                        this.refMap[key] = { current: null };
                    if (isNil(this.resolveFuncMap[key]))
                        this.resolveFuncMap[key] = [];
                    const resolveFunc = () => resolve(this.refMap[key].current);
                    this.resolveFuncMap[key].push(resolveFunc);
                    // timeout
                    if (timeout && timeout > 0) {
                        const timer = setTimeout(() => {
                            var _a;
                            const index = (_a = this.resolveFuncMap[key]) === null || _a === void 0 ? void 0 : _a.findIndex(item => item === resolveFunc);
                            if (~index) {
                                this.resolveFuncMap[key][index] = null;
                                reject(new Error("timeout"));
                            }
                        }, timeout);
                        if (isNil(this.rejectTimerMap[key]))
                            this.rejectTimerMap[key] = [];
                        this.rejectTimerMap[key].push(timer);
                    }
                }
            });
        };
        this.setRef = (key, value) => {
            this.refMap[key] ? (this.refMap[key].current = value) : (this.refMap[key] = { current: value });
        };
    }
}
export default RefPromiseHelper;
