import dayjs from "dayjs";
import duration from "dayjs/plugin/duration";
dayjs.extend(duration);
class StorageHelper {
    constructor(prefix) {
        this.length = localStorage.length;
        this.setItem = (key, value, _expire) => {
            if (this.prefix)
                key = `${this.prefix}_${key}`;
            _expire = this.calcExpire(_expire);
            const res = { data: value, _expire };
            localStorage.setItem(String(key), JSON.stringify(res));
        };
        this.removeItems = (keys) => {
            keys.forEach(item => {
                if (this.prefix)
                    item = `${this.prefix}_${item}`;
                localStorage.removeItem(String(item));
            });
        };
        this.removeItem = localStorage.removeItem.bind(localStorage);
        this.clear = localStorage.clear.bind(localStorage);
        this.key = localStorage.key.bind(localStorage);
        this.prefix = prefix;
    }
    calcExpire(duration) {
        if (!duration)
            return Number.MAX_SAFE_INTEGER;
        if (typeof duration === "number")
            return Date.now() + duration;
        const milliseconds = dayjs.duration(duration).asMilliseconds();
        return milliseconds ? Date.now() + milliseconds : Number.MAX_SAFE_INTEGER;
    }
    getItem(key, defaultValue) {
        var _a, _b, _c;
        if (this.prefix)
            key = `${this.prefix}_${key}`;
        let res = localStorage.getItem(String(key));
        if (res) {
            try {
                res = JSON.parse(res !== null && res !== void 0 ? res : "null");
            }
            catch (e) { }
        }
        if (typeof res === "object" && typeof (res === null || res === void 0 ? void 0 : res._expire) === "number" && (res === null || res === void 0 ? void 0 : res.data)) {
            if (res._expire < Date.now()) {
                localStorage.removeItem(String(key));
                return defaultValue !== null && defaultValue !== void 0 ? defaultValue : null;
            }
            return (_b = (_a = res === null || res === void 0 ? void 0 : res.data) !== null && _a !== void 0 ? _a : defaultValue) !== null && _b !== void 0 ? _b : null;
        }
        else
            return (_c = res !== null && res !== void 0 ? res : defaultValue) !== null && _c !== void 0 ? _c : null;
    }
}
export default StorageHelper;
